package fr.openium.consentium_ui.ui.components

import android.widget.Toast
import androidx.activity.compose.BackHandler
import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import fr.openium.consentium.api.Consentium
import fr.openium.consentium.api.state.SetConsentiumState
import fr.openium.consentium_ui.R
import fr.openium.consentium_ui.ui.ConsentiumScreen
import fr.openium.consentium_ui.ui.ConsentiumUIViewModel
import fr.openium.consentium_ui.ui.adapter.toAcceptedPurposeChoices
import fr.openium.consentium_ui.ui.adapter.toDeniedPurposeChoices
import fr.openium.consentium_ui.ui.adapter.toPurposeChoices
import fr.openium.consentium_ui.ui.components.style.ConsentiumColors
import fr.openium.consentium_ui.ui.components.style.ConsentiumDefaults
import fr.openium.consentium_ui.ui.components.style.ConsentiumTypography
import fr.openium.consentium_ui.ui.components.style.LocalColors
import fr.openium.consentium_ui.ui.components.style.LocalTypography
import fr.openium.consentium_ui.ui.model.ConsentiumPageUI
import fr.openium.consentium_ui.ui.model.LoadingElement
import kotlinx.coroutines.launch

@Composable
fun ConsentiumComponent(
    consentium: Consentium,
    onQuitConsent: () -> (Unit),
    colors: ConsentiumColors = ConsentiumDefaults.colors(),
    typography: ConsentiumTypography = ConsentiumDefaults.typography(),
    defaultLandingPage: ConsentiumPageUI = ConsentiumPageUI.GENERAL_CONSENT,
) {
    // Property
    val viewModel: ConsentiumUIViewModel = hiltViewModel()
    val loadingState by viewModel.state.collectAsStateWithLifecycle()
    var currentPage by remember(defaultLandingPage) { mutableStateOf(defaultLandingPage) }
    val scope = rememberCoroutineScope()
    val context = LocalContext.current
    val errorMessage = stringResource(id = R.string.save_consents_error_message)
    val consentiumState by consentium.saveConsentState.collectAsStateWithLifecycle()
    var loadingElement by remember { mutableStateOf(LoadingElement.NONE) }

    // Effect
    LaunchedEffect(Unit) {
        viewModel.init(apiKey = consentium.apiKey, appId = consentium.appId)
    }

    LaunchedEffect(consentiumState) {
        when (consentiumState) {
            SetConsentiumState.Idle,
            SetConsentiumState.Loading,
                -> {
            }

            SetConsentiumState.Error -> {
                Toast.makeText(context, errorMessage, Toast.LENGTH_SHORT).show()
            }

            SetConsentiumState.Success -> {
                onQuitConsent()
            }
        }
    }

    BackHandler {
        when {
            defaultLandingPage == ConsentiumPageUI.GENERAL_CONSENT && currentPage == ConsentiumPageUI.DETAILS_CONSENT -> {
                currentPage = ConsentiumPageUI.GENERAL_CONSENT
            }

            currentPage == ConsentiumPageUI.COOKIES_POLICY -> {
                currentPage = ConsentiumPageUI.GENERAL_CONSENT
            }

            else -> {
                onQuitConsent()
            }
        }
    }

    // View
    CompositionLocalProvider(
        LocalColors provides colors,
        LocalTypography provides typography,
    ) {
        ConsentiumScreen(
            state = loadingState,
            page = currentPage,
            loadingElement = if (consentiumState is SetConsentiumState.Loading) loadingElement else LoadingElement.NONE,
            onNavigateBack = {
                when {
                    defaultLandingPage == ConsentiumPageUI.GENERAL_CONSENT && currentPage == ConsentiumPageUI.DETAILS_CONSENT -> {
                        currentPage = ConsentiumPageUI.GENERAL_CONSENT
                    }

                    else -> {
                        onQuitConsent()
                    }
                }
            },
            onNavigateToDetails = {
                currentPage = ConsentiumPageUI.DETAILS_CONSENT
            },
            onAcceptAndClose = { consent ->
                scope.launch {
                    loadingElement = LoadingElement.BUTTON
                    consentium.saveConsents(consent.toAcceptedPurposeChoices())
                }
            },
            onDenyAndClose = { consent ->
                scope.launch {
                    loadingElement = LoadingElement.LINK
                    consentium.saveConsents(consent.toDeniedPurposeChoices())
                }
            },
            onSaveAndCloseDetails = { consent ->
                scope.launch {
                    loadingElement = LoadingElement.BUTTON
                    consentium.saveConsents(consent.toPurposeChoices())
                }
            },
            onClickCookiesPolicies = {
                currentPage = ConsentiumPageUI.COOKIES_POLICY
            },
            onClickRetry = {
                viewModel.init(apiKey = consentium.apiKey, appId = consentium.appId)
            }
        )
    }
}
